/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2010-2014 Intel Corporation
 */

#ifndef __INCLUDE_RTE_LRU_MIPS_H__
#define __INCLUDE_RTE_LRU_MIPS_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>

#include <rte_config.h>
#include <rte_vect.h>

#ifndef RTE_TABLE_HASH_LRU_STRATEGY
#define RTE_TABLE_HASH_LRU_STRATEGY                        3
#endif

#if RTE_TABLE_HASH_LRU_STRATEGY == 3

#define lru_init(bucket)						\
	{ bucket->lru_list = ~0LLU; }

static inline int
f_lru_pos(uint64_t lru_list)
{
	xmm_t lst = (xmm_t) {.i64 = { lru_list, (uint64_t)-1} };
	uint16_t min_val, i, j = 0;

	min_val = lst.u16[0];
	for (i = 0; i < 8; i++) {
		if (min_val > lst.u16[i]) {
			min_val = lst.u16[i];
			j = i;
		}
	}

	return j;
}
#define lru_pos(bucket) f_lru_pos(bucket->lru_list)

#ifdef RTE_ARCH_NO_VECTOR
#define lru_update(bucket, mru_val)					\
do {									\
	const uint64_t orvals[] = {0xFFFFLLU, 0xFFFFLLU << 16,		\
		0xFFFFLLU << 32, 0xFFFFLLU << 48, 0LLU};		\
	const uint64_t decs[] = {0x1000100010001LLU, 0};		\
	int j = 0;							\
	xmm_t v_zeros = (xmm_t) {.u64 = {0, 0} };			\
	xmm_t lru = v_zeros, vdec = v_zeros;				\
	lru.i64[0] = bucket->lru_list;					\
	vdec.i64[0] = decs[mru_val>>2];					\
	for (j = 0; j < 8; j++)						\
		lru.u16[j] -= vdec.u16[j];				\
	bucket->lru_list = lru.i64[0] | orvals[mru_val];		\
} while (0)
#endif
#endif

#ifdef __cplusplus
}
#endif

#endif
